function [C, coord, onering] = calc_allc_poly_fit2d(v, patch)
% Compute the one-ring and related quantities.
% 
% If you use this code in your own work, please cite the following paper:
% [1] T. W. Meng, G. P.-T. Choi and L. M. Lui, 
%     "TEMPO: Feature-Endowed Teichmller Extremal Mappings of Point Clouds."
%     SIAM Journal on Imaging Sciences, 9(4), pp. 1922-1962, 2016.
%
% Copyright (c) 2015-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

k = patch.k;
n = size(v,1);
knnInd = patch.knnInd;
weight_id = patch.weight_id;

[coeff, powerc] = calc_coeff_power(patch.order, 0,0);
C = zeros(size(v,1), size(coeff,1) * k);
coord = zeros(n, k*2);

v_local_1 = reshape(v(knnInd,1), [n,k]);
v_local_2 = reshape(v(knnInd,2), [n,k]);

onering = ones(n, 3*k);

for j=1:n
    knn = zeros(k,2);
    % center to v(j,:)
    knn(:,1) = v_local_1(j,:)' - v(j,1);
    knn(:,2) = v_local_2(j,:)' - v(j,2);
    
    x_ = knn(:,1);
    y_ = knn(:,2);
    coord(j,:) = [x_; y_]';
    
    norm = x_.^2+ y_.^2;
    maxnorm = max(norm);

    switch(weight_id)
        case 1
            weight = exp(-sqrt(k)*norm/maxnorm); %gaussian
        case 2
            weight = exp(-sqrt(k)*norm/maxnorm)/k; %gaussian c1
        case 3
            weight = exp(-k*norm/maxnorm/2); %gaussian c2
    end
    
    weight(1) = 1;
    
    A = zeros(length(coeff), length(coeff));
    B = zeros(length(coeff), k);
    for i=1:k
        b = coeff .* (x_(i).^powerc(:,1)) .* (y_(i).^powerc(:,2));
        B(:,i) = weight(i) * b;
        A = A + B(:,i) * b';
    end

    c = A\B;
    C(j,:) = c(:)';
    
    % calc one ring
    TR = delaunay(x_,y_);
    [row,~] = find(TR==1);
    onering_ = TR(row,:);
    
    if numel(onering_) > size(onering, 2)
        onering = [onering, ones(n, numel(onering_) - size(onering, 2))];
    end
    onering(j, 1:numel(onering_)) = reshape(onering_.', [numel(onering_), 1]);
end
end