function [coeff, powerc] = calc_coeff_power(order, diff_order_x, diff_order_y)
% Compute the coefficients for Laplacian approximation.
% 
% If you use this code in your own work, please cite the following paper:
% [1] T. W. Meng, G. P.-T. Choi and L. M. Lui, 
%     "TEMPO: Feature-Endowed Teichmller Extremal Mappings of Point Clouds."
%     SIAM Journal on Imaging Sciences, 9(4), pp. 1922-1962, 2016.
%
% Copyright (c) 2015-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

coeff = ones( (order+2) * (order+1) / 2, 1);
powerc = zeros( (order+2) * (order+1) / 2, 2);
for i=0:order
    for j=0:i
        powerc( (i+1)*i/2 + j + 1, :) = [i-j, j];
    end
end

if diff_order_x > 0 || diff_order_y > 0
    for i=0:order
        for j=0:i
            count = (i+1)*i/2 + j + 1;
            for l=0:diff_order_x-1
                coeff(count, 1) = coeff(count, 1) * (powerc(count,1)-l);
            end
            if powerc(count, 1) < diff_order_x
                powerc(count,1) = 0;
            else
                powerc(count, 1) = powerc(count, 1) -diff_order_x;
            end
            
            for l=0:diff_order_y-1
                coeff(count, 1) = coeff(count, 1) * (powerc(count,2)-l);
            end
            if powerc(count, 2) < diff_order_y
                powerc(count,2) = 0;
            else
                powerc(count, 2) = powerc(count, 2) -diff_order_y;
            end
        end
    end
end
end