function patch = calc_patch_polyfit(v, patch)
% Update patch properties.
% 
% If you use this code in your own work, please cite the following paper:
% [1] T. W. Meng, G. P.-T. Choi and L. M. Lui, 
%     "TEMPO: Feature-Endowed Teichmller Extremal Mappings of Point Clouds."
%     SIAM Journal on Imaging Sciences, 9(4), pp. 1922-1962, 2016.
%
% Copyright (c) 2015-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

if ~isfield(patch, 'k')
    patch.k = 15;
end
if ~isfield(patch, 'order')
    patch.order = 2;
end
if ~isfield(patch, 'weight_id')
    patch.weight_id = 2;
end

patch.knnInd = knnsearch(v, v, 'K', patch.k);

[C, coord, onering] = calc_allc_poly_fit2d(v, patch);
patch.C = C;
patch.coord = coord;
patch.onering = onering;

patch.Differential = differential(patch);
end

function D = differential(patch)
knnInd = patch.knnInd;
C = patch.C;
k = size(knnInd,2);

D.dx = C(:, 2:6:6*k-4);
D.dy = C(:, 3:6:6*k-3);
D.dxx = 2.*C(:, 4:6:6*k-2);
D.dxy = C(:, 5:6:6*k-1);
D.dyy = 2.*C(:, 6:6:6*k);
end